
/******************************************************************************
* MODULE     : integer.gen.cc
* DESCRIPTION: integers based on gmp
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Arithmetic/integer.gen.h>

#module code_integer
#import integer

/******************************************************************************
* conversion
******************************************************************************/

integer::integer () {
  rep= new integer_rep();
}

integer::integer (int i) {
  rep= new integer_rep();
  mpz_set_si (rep->I, i);
}

integer::integer (string s, int base) {
  rep= new integer_rep();
  char* ptr= as_charp (s);
  mpz_set_str (rep->I, ptr, base);
  delete[] ptr;
}

integer::operator string () {
  int n= mpz_sizeinbase (rep->I, 10);
  char s[n+2]; s[n]= s[n+1]= '\0';
  mpz_get_str (s, 10, rep->I);
  return string (s);
}

integer::operator tree () {
  return tree ((string) (*this));
}

ostream&
operator << (ostream& out, integer i) {
  return out << ((string) i);
}

/******************************************************************************
* basic arithmetics
******************************************************************************/

integer
operator + (integer i, integer j) {
  integer k;
  mpz_add (k->I, i->I, j->I);
  return k;
}

integer
operator - (integer i) {
  integer j;
  mpz_neg (j->I, i->I);
  return j;
}

integer
operator - (integer i, integer j) {
  integer k;
  mpz_sub (k->I, i->I, j->I);
  return k;
}

integer
operator * (integer i, integer j) {
  integer k;
  mpz_mul (k->I, i->I, j->I);
  return k;
}

integer
operator / (integer i, integer j) {
  integer k;
  mpz_fdiv_q (k->I, i->I, j->I);
  return k;
}

integer
operator % (integer i, integer j) {
  integer k;
  mpz_fdiv_r (k->I, i->I, j->I);
  return k;
}

integer
gcd (integer i, integer j) {
  integer k;
  mpz_gcd (k->I, i->I, j->I);
  return k;
}

/******************************************************************************
* tests on sign
******************************************************************************/

int
sign (integer i) {
  return mpz_sgn (i->I);
}

bool
operator == (integer i, integer j) {
  return mpz_cmp (i->I, j->I) == 0;
}

bool
operator != (integer i, integer j) {
  return mpz_cmp (i->I, j->I) != 0;
}

bool
operator <= (integer i, integer j) {
  return mpz_cmp (i->I, j->I) <= 0;
}

bool
operator >= (integer i, integer j) {
  return mpz_cmp (i->I, j->I) >= 0;
}

bool
operator < (integer i, integer j) {
  return mpz_cmp (i->I, j->I) < 0;
}

bool
operator > (integer i, integer j) {
  return mpz_cmp (i->I, j->I) > 0;
}

#endmodule // code_integer
