
/******************************************************************************
* MODULE     : relaxed_test.gen.cc
* DESCRIPTION: test for relaxed power series
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <timer.gen.h>
#include <Arithmetic/int_mod.gen.cc>
#include <Arithmetic/rational.gen.h>
#include <Arithmetic/fp_real.gen.h>
// #include <Univariate/Relaxed/relaxed.gen.cc>
#include <Univariate/Truncated/truncated.gen.cc>

#module code_relaxed_test
#import timer
#import rational
#import fp_real
#import code_int_mod (1234577)
#import code_relaxed (int_mod<1234577>)

#define REPS         1
#define BITS         10000
#define CST          int_mod<1234577>
#define SERIES       series<int_mod<1234577>>
#define ZSERIES      zealous<int_mod<1234577>>
#define IMPL_SERIES  implicit_series<int_mod<1234577>>
#define SET_TERMS    set_series<int_mod<1234577>>_precision
#define MAX_TIME     3600000
#ifdef has_divisible<int_mod<1234577>>
#define divisible
#endif

timer tm;

void
test1 (int TERMS) {
  SERIES I (CST (1)), z (1, 1);
  SET_TERMS (TERMS);
  cout << "I= " << I << "\n";
  cout << "z= " << z << "\n";
  cout << "I+z= " << I+z << "\n";
  cout << "I-z= " << I-z << "\n";
  cout << "z*z= " << z*z << "\n";
  cout << "I/(I+z)= " << I/(I+z) << "\n";
#ifdef divisible
  cout << "exp z  = " << exp (z) << "\n";
  cout << "I/exp z= " << I/exp(z) << "\n";
  cout << "log(I+z)= " << log(I+z) << "\n";
  cout << "exp(z*exp(z))= " << exp(z*exp(z)) << "\n";
  cout << "exp(z*exp(z))= " << compose (exp(z), z*exp(z)) << "\n";
#endif
}

#ifdef divisible
void
test2 (int TERMS) {
  SERIES I (CST (1)), z (1, 1);
  SET_TERMS (TERMS);
  // cout << Mahler  (exp(z), 2) << "\n";
  // cout << compose (exp(z), (z+z*z)(0,3)) << "\n";
  // cout << compose (exp(z), z+z*z) << "\n";
  cout << exp(z*exp(z)) << "\n";
  cout << compose (exp(z), z*exp(z)) << "\n";
  // cout << exp (z/(I-z)) << "\n";
  // cout << compose (exp(z), z/(I-z)) << "\n";
  // cout << compose (exp(z), z(0,2), (I-z)(0,2)) << "\n";
}
#endif

#ifdef divisible
void
test3 (int TERMS) {
  SERIES I (CST (1)), z (1, 1);
  SERIES f= exp (z* exp (z));
  f->set_precision (TERMS);

  int i;
  for (i=0; i<TERMS; i++) {
    cerr << "[" << i << "]     \r";
    CST c= ((SERIES) f)[i];
    (void) c;
    if (tm->watch() > MAX_TIME) { cout << "\n"; return; }
  }
}
#endif

CST cst (int n) { (void) n; return 1; }

void
test4 (int TERMS) {
  int i;
  SERIES I (CST (1)), z (1, 1);
  IMPL_SERIES f (1);
  f(0)= CST (1);
  f->eq= ((f*f*f + CST(2)*Mahler(f,3)) << 1) / CST(3);
  f->set_precision (TERMS);
  // cout << TERMS << ": " << ((SERIES) f) (0,TERMS) << "\n";
  // for (i=0; i<TERMS; i++) cout << i << ":\t" << f[i] << "\n";

  for (i=0; i<TERMS; i++) {
    cerr << "[" << i << "]     \r";
    CST c= ((SERIES) f)[i];
    (void) c;
    if (tm->watch() > MAX_TIME) { cout << "\n"; return; }
  }

  f->eq= SERIES();
}

void
test5 (int TERMS) {
  int i;
  SERIES I (CST (1)), z (1, 1);
  IMPL_SERIES f (1);
  f(0)= CST (0);
  SERIES df= d(f);
  // f->eq= (I+ compose (f, z(0,2), (I+z)(0,2))- ((df*df)<<4)) << 1;
  f->eq= (I+ compose (f, z/(I+z))- ((df*df)<<4)) << 1;
  // f(z)= z (1+ f (z/(1+z))- z^4 f'(z)^2)
  f->set_precision (TERMS);
  // cout << TERMS << ": " << ((SERIES) f) (0,TERMS) << "\n";
  // for (i=0; i<TERMS; i++) cout << i << ":\t" << f[i] << "\n";

  for (i=0; i<TERMS; i++) {
    cerr << "[" << i << "]     \r";
    CST c= ((SERIES) f)[i];
    (void) c;
    if (tm->watch() > MAX_TIME) { cout << "\n"; return; }
  }

  f->eq= SERIES();
}

void
test6 (int TERMS) {
  SERIES  I (CST (1)), z (1, 1);
  ZSERIES g (4); g[2]= g[3]= CST(1);
  IMPL_SERIES f (1);
  f(0)= CST (0);
  f->eq= z + (((I+z) * compose (f>>1, g)) << 2);
  // f->eq= z + (((I+z) * compose (f>>1, SERIES (g))) << 2);
  // f(z)= z + f(z^2 + z^3)
  f->set_precision (TERMS);
  // cout << TERMS << ": " << ((SERIES) f) (0,TERMS) << "\n";

  int i;
  for (i=0; i<TERMS; i++) {
    cerr << "[" << i << "]     \r";
    CST c= ((SERIES) f)[i];
    (void) c;
    if (tm->watch() > MAX_TIME) { cout << "\n"; return; }
  }

  f->eq= SERIES();
}

#ifdef divisible
void
test7 (int TERMS) {
  SERIES  z (1, 1);
  IMPL_SERIES f (3);
  f(0)= f(1)= CST (1);
  f->eq= I(I(I(f*f+f)));
  // f'''(z)= f^2+ f, f(0)=1, f'(0)=1, f''(0)=0
  f->set_precision (TERMS);
  // cout << TERMS << ": " << ((SERIES) f) (0,TERMS) << "\n";

  int i;
  for (i=0; i<TERMS; i++) {
    cerr << "[" << i << "]     \r";
    CST c= ((SERIES) f)[i];
    (void) c;
    if (tm->watch() > MAX_TIME) { cout << "\n"; return; }
  }

  f->eq= SERIES();
}
#endif

int
bench (void (*f) (int), int TERMS) {
  int i;
  tm->start ();
  for (i=0; i< REPS; i++) f (TERMS);
  int millisec= tm->stop ()/REPS;
  cout << "Terms= " << TERMS << ": \t" << millisec << " ms\n";
  return millisec;
}

void
bench (void (*f) (int)) {
  /*
  if (bench (f, 1) > MAX_TIME) return;
  if (bench (f, 2) > MAX_TIME) return;
  if (bench (f, 5) > MAX_TIME) return;
  */
  if (bench (f, 10) > MAX_TIME) return;
  if (bench (f, 20) > MAX_TIME) return;
  if (bench (f, 50) > MAX_TIME) return;
  if (bench (f, 100) > MAX_TIME) return;
  if (bench (f, 200) > MAX_TIME) return;
  if (bench (f, 500) > MAX_TIME) return;
  if (bench (f, 1000) > MAX_TIME) return;
  if (bench (f, 2000) > MAX_TIME) return;
  if (bench (f, 5000) > MAX_TIME) return; 
  if (bench (f, 10000) > MAX_TIME) return;
  if (bench (f, 20000) > MAX_TIME) return;
  if (bench (f, 50000) > MAX_TIME) return;
  if (bench (f, 100000) > MAX_TIME) return;
  if (bench (f, 200000) > MAX_TIME) return;
  if (bench (f, 500000) > MAX_TIME) return;
  if (bench (f, 1000000) > MAX_TIME) return;
}

int
main () {
  cout << "Testing relaxed power series\n";
  cout << " | int_mod<1234577> coefficients\n";
  // cout << " | Brent and Kung composition\n";
  cout << " | Truncated multiplication\n";
  cout << " | Polya equation\n\n";
  set_fp_real_precision (BITS);
  bench (test4);
  return 0;
}

#endmodule // relaxed_test
