
/******************************************************************************
* MODULE     : tex_test.gen.cc
* DESCRIPTION: Display tex fonts on screen
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <dir.gen.h>
#include <font.gen.h>
#include <window.gen.h>
#include <Widget/basic_widget.gen.h>
#include <font.gen.h>

#module code_tex_test
#import dir
#import font
#import window
#import basic_widget
#import font

/******************************************************************************
* The widget
******************************************************************************/

class font_widget_rep: public basic_widget_rep {
  int  sfactor;
  int  pixel;
  font text_fn;
  font fn;

  int  fn_height;
  int  fn_depth;

public:
  font_widget_rep (display dis, int sf, font text_fn, font fn);
  operator tree ();
  void handle_get_size (get_size_event ev);
  void handle_repaint (repaint_event ev);
};

/******************************************************************************
* Setup game
******************************************************************************/

font_widget_rep::font_widget_rep (display dis, int sfactor2, font f1, font f2):
  basic_widget_rep (dis), sfactor (sfactor2), pixel (sfactor2*PIXEL),
  text_fn (f1), fn (f2)
{
  text_extents ex;
  text_fn->get_extents ("f", ex);
  fn_height= ex->y2-ex->y1;
  fn_depth = -ex->y1;
}

font_widget_rep::operator tree () {
  return "tex_test";
}

void
font_widget_rep::handle_get_size (get_size_event ev) {
  ev->w= 400*PIXEL;
  ev->h= 300*PIXEL;
}

/******************************************************************************
* Repainting
******************************************************************************/

void
font_widget_rep::handle_repaint (repaint_event ev) {
  win->set_background (dis->white);
  win->clear (ev->x1, ev->y1, ev->x2, ev->y2);
  win->set_color (dis->black);
  win->set_shrinking_factor (sfactor);

  int y, y1, y2;
  y1= ((ev->y1*sfactor)/fn_height) - 1;
  y2= ((ev->y2*sfactor)/fn_height) + 1;
  for (y=y1; y<=y2; y++) {
    int ypos= y*fn_height+ fn_depth;
    // fn->var_draw (win, "T", 0, ypos);
    fn->var_draw (win, "This is a test to see how fast a TeX font is displayed on the screen. We can not expect TeXmacs to scroll faster than this window.", 0, ypos);
  }

  win->set_shrinking_factor (1);
}

/******************************************************************************
* The main program
******************************************************************************/

int
main (int argc, char** argv) {
  if ((argc<3) || (argc>6)) {
    cout << "Usage: xtexfont <family> <size>\n";
    cout << "       xtexfont <family> <size> <dpi>\n";
    cout << "       xtexfont <family> <size> <dpi> <sfactor>\n";
    cout << "       xtexfont <family> <size> <dpi> <sfactor> <design size>\n";
    return 0;
  }
  string name= argv[1];
  int size   = as_int (string (argv[2]));
  int dpi    = argc>3? as_int (string (argv[3])): 600;
  int sfactor= argc>4? as_int (string (argv[4])): 5;
  int dsize  = argc>5? as_int (string (argv[5])): 10;

  install_texmacs ();
  install_tex ();
  debug (0, TRUE);
  debug (2, TRUE);
  display dis= open_display (argc, argv);
  font text_fn= tex_font (dis, "cmr", size, dpi, 10);
  font fn     = tex_font (dis, name, size, dpi, dsize);
  widget wid= canvas_widget (new font_widget_rep (dis, sfactor, text_fn, fn));
  wid << set_extents (0, -3000*PIXEL, 800*PIXEL, 0);
  window win= plain_window (wid, "tex_test");
  win->map ();
  dis->event_loop ();
  close_display (dis);
  return 0;
}

#endmodule // code_tex_test
